/*
 *  Jomar3D, a lobster-running-to-catch game.
 *  Copyright (C) 2006-2008 Jonathan Michalon <studios.chalmion@no-log.org>.
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <GL/glut.h>

#include "draw.h"
#include "interface.h"
#include "jomar3D.h"
#include "scores.h"
#include "utils.h"

/* load all scores from file */

void
loadscores (void)
{
  FILE *file;
  int i, j;
  char
    tmpget[max ((ncharmaxint + 1) * NSTORE + 2, MAXCNAME + ncharmaxint + 2)];

  if (!(file = fopen (SCOREFILE, "r")))
    {
      perror ("fopen for read");
      printf ("Cannot open %s... no load of the scores!\n", SCOREFILE);
      return;
    }
  for (i = 0;; i++)
    {
//  get the next record
      fgets (&tmpget[0],
             max ((ncharmaxint + 1) * NSTORE + 2, MAXCNAME + ncharmaxint + 2),
             file);
      if (feof (file) || i > 3 * NSTORE + 1)
        break;
//  store the name and the number of games done
      if ((float) (i - 1) / 3. == (float) ((i - 1) / 3))
        {
          strncpy (&names[(i - 1) / 3][0], &tmpget[0], MAXCNAME);
          for (j = MAXCNAME; tmpget[j] == ' '; j--);
          names[(i - 1) / 3][j + 1] = '\0';
          games[(i - 1) / 3] = atoi (&tmpget[MAXCNAME]);
        }
//  store the times needed
      if ((float) (i - 2) / 3. == (float) ((i - 2) / 3))
        for (j = 0; j < NSTORE; j++)
          {
            times[(i - 2) / 3][j] = atoi (&tmpget[j * (ncharmaxint + 1)]);
          }
//  store the steps needed
      if ((float) (i - 3) / 3. == (float) ((i - 3) / 3) && i != 0)
        for (j = 0; j < NSTORE; j++)
          {
            steps[(i - 3) / 3][j] = atoi (&tmpget[j * (ncharmaxint + 1)]);
          }
    }
  fclose (file);
}

/* write the scores into the file */

void
writescores (void)
{
  FILE *file;
  int i, j;

  if (!(file = fopen (SCOREFILE, "w")))
    {
      perror ("fopen for write");
      printf ("Cannot open %s... no save of the scores!\n", SCOREFILE);
      return;
    }
  fprintf (file,
           "##### This is the storage file of the scores of jomar3D. #####\n");
  for (i = 0; i < MAXNAMES; i++)
    {
      if (strlen (names[i]) == 0)
        break;
      fprintf (file, "%-*s %d\n", MAXCNAME, names[i], games[i]);
      for (j = 0; j < NSTORE; j++)
        fprintf (file, "%*lu ", ncharmaxint, times[i][j]);
      fprintf (file, "\n");
      for (j = 0; j < NSTORE; j++)
        fprintf (file, "%*lu ", ncharmaxint, steps[i][j]);
      fprintf (file, "\n");
    }
  fclose (file);
}

/* validate the current entered name as given */

void
validatename (void)
{
  int i, j, k, l;
  long unsigned int m;
  int place = 0;

#if (DEBUG)
  printf ("Values to compare: %d ticks %d steps.\n", timeneeded, martsteps);
#endif

  for (i = 0; i < MAXNAMES; i++)
    {
      if (strncmp (name, names[i], MAXCNAME) == 0)
        {
#if (DEBUG)
          printf ("Known name.\n");
#endif
          output (750., 1400., 0.3, 0.3, text[uselangnb][33]);
          output (750., 1000., 0.3, 0.3, text[uselangnb][36]);
          games[i] += 1;
          for (j = 0; j < NSTORE; j++)
            {
              if (times[i][j] == 0)
                {
#if (DEBUG)
                  printf ("Not enough stat in times.\n");
#endif
                  output (750., 1300., 1.0, 0.5, text[uselangnb][32]);
                  times[i][j] = timeneeded;
                  goto next;
                }
            }
          for (j = 0; j < NSTORE; j++)
            {
              if (times[i][j] > timeneeded)
                {
#if (DEBUG)
                  printf ("Better value in times.\n");
#endif
                  output (750., 1300., 1.0, 0.5, text[uselangnb][34]);
                  for (k = 0, l = 0, m = 0; k < NSTORE; k++)
                    if (m < times[i][k])
                      {
                        m = times[i][k];
                        l = k;
                      }
                  times[i][l] = timeneeded;
                  goto next;
                }
            }
#if (DEBUG)
          printf ("Nothing to say for time.\n");
#endif
          output (750., 1300., 1.0, 0.5, text[uselangnb][35]);
        next:;
          for (j = 0; j < NSTORE; j++)
            {
              if (steps[i][j] == 0)
                {
#if (DEBUG)
                  printf ("Not enough stat in steps.\n");
#endif
                  output (750., 900., 1.0, 0.5, text[uselangnb][32]);
                  steps[i][j] = martsteps;
                  goto write;
                }
            }
          for (j = 0; j < NSTORE; j++)
            {
              if (steps[i][j] > martsteps)
                {
#if (DEBUG)
                  printf ("Better value in steps.\n");
#endif
                  output (750., 900., 1.0, 0.5, text[uselangnb][37]);
                  for (k = 0, l = 0, m = 0; k < NSTORE; k++)
                    if (m < steps[i][k])
                      {
                        m = steps[i][k];
                        l = k;
                      }
                  steps[i][l] = martsteps;
                  goto write;
                }
            }
#if (DEBUG)
          printf ("Nothing to say for step.\n");
#endif
          output (750., 900., 1.0, 0.5, text[uselangnb][38]);
          goto write;
        }
      if (strlen (names[i]) != 0 && strncmp (name, names[i], MAXCNAME) != 0)
        continue;
#if (DEBUG)
      printf ("New name.\n");
#endif
      output (750., 700., 1.0, 1.0, text[uselangnb][30]);
      strcpy (names[i], name);
      games[i] = 1;
      times[i][0] = timeneeded;
      for (j = 1; j < NSTORE; j++)
        times[i][j] = 0;
      steps[i][0] = martsteps;
      for (j = 1; j < NSTORE; j++)
        steps[i][j] = 0;
      place = 1;
      break;
    }
  if (!place)
    printf ("No place more for a new name is the tabular - grow MAXNAMES.\n");

write:;
  writescores ();
  glutKeyboardFunc (NULL);
  glutMouseFunc (NULL);
  glutTimerFunc (8000, changemenu, MP);
  validatenextredisplay = 0;
}

/* keyboard function for the score interface */

void
doscores (unsigned char c, int x, int y)
{
  (void) x, (void) y;
  switch (c)
    {
    case 27:                   /* Escape */
      exit (0);
      break;
    case 13:                   /* Validate */
      validatenextredisplay = 1;
      break;
    }
  if (c == '\b')
    {
      name[strlen (name) - 1] = '\0';
      name[strlen (name)] = 0;
    }
  else
    {
      if (((int) c > 31 && (int) c < 128) || (int) c == 176)   // select the stored caracters
        if (strlen (name) < MAXCNAME)
          strncat (&name[0], (char *) &c, 1);
    }
  glutPostRedisplay ();
#if (DEBUG)
  printf ("%s | %c %d\n", name, c, c);
#endif
}

/* mouse interface for the scores (select name from list) */

void
selname (int button, int state, int x, int y)
{
  float xd = (float) (x) / (float) (winwidth) * 1500., yd =
    1500. - (float) (y) / (float) (winheight) * 1500.;
  int playernb = -1;

  (void) button;
#if (DEBUG)
  printf ("xd, yd %f %f\n", xd, yd);
#endif
  if (state != GLUT_DOWN)
    return;
  if (xd > 1400. || xd < 100.)
    return;
  if (yd > 1050. || yd < 300.)
    return;
  playernb = (int) ((1050. - yd) / (700. / MAXNAMES));
#if (DEBUG)
  printf ("selected player number %d\n", playernb);
#endif
  if (strlen (names[playernb]) == 0)
    return;
  strncpy (&name[0], &names[(int) ((1050. - yd) / (700. / MAXNAMES))][0],
           MAXCNAME);
  doscores (13, 0, 0);
}

/* begin the score interface */

void
beginscores (int dummy)
{
  int i;

  (void) dummy;
  glutKeyboardFunc (doscores);
  glutKeyboardUpFunc (NULL);
  glutSpecialFunc (NULL);
  glutSpecialUpFunc (NULL);
  glutMouseFunc (selname);
  loadscores ();
  for (i = 0; i < MAXCNAME; i++)
    name[i] = 0;
  validatenextredisplay = 0;
  glutPostRedisplay ();
}
